package it.inaf.ia2.gms.service;

import io.zonky.test.db.AutoConfigureEmbeddedDatabase;
import it.inaf.ia2.gms.model.Permission;
import it.inaf.ia2.gms.model.RapUser;
import it.inaf.ia2.gms.model.UserPermission;
import it.inaf.ia2.gms.persistence.GroupsDAO;
import it.inaf.ia2.gms.persistence.PermissionsDAO;
import it.inaf.ia2.gms.persistence.model.GroupEntity;
import it.inaf.ia2.gms.persistence.model.PermissionEntity;
import it.inaf.ia2.gms.rap.RapClient;
import java.util.Collections;
import java.util.List;
import static org.junit.Assert.assertEquals;
import org.junit.Test;
import org.junit.runner.RunWith;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;

@RunWith(SpringRunner.class)
@AutoConfigureEmbeddedDatabase(beanName = "dataSource")
public class PermissionsServiceIntegrationTest {

    private static final String USER_ID = "USER_ID";

    @Autowired
    private GroupsDAO groupsDAO;

    @Autowired
    private PermissionsDAO permissionsDAO;

    @MockBean
    private RapClient rapClient;

    @Test
    public void permissionsRetrievalTest() {

        // Mock RAP client
        RapUser rapUser = new RapUser();
        rapUser.setId(USER_ID);
        when(rapClient.getUsers(any())).thenReturn(Collections.singletonList(rapUser));

        PermissionsService permissionsService = new PermissionsService(permissionsDAO, rapClient);

        // Create root
        GroupEntity root = new GroupEntity();
        root.setId(GroupsService.ROOT);
        root.setName(GroupsService.ROOT);
        root.setPath("");
        root = groupsDAO.createGroup(root);

        PermissionEntity superAdminPermission = new PermissionEntity();
        superAdminPermission.setUserId(USER_ID);
        superAdminPermission.setGroupId(root.getId());
        superAdminPermission.setPermission(Permission.ADMIN);
        superAdminPermission.setGroupPath(root.getPath());
        permissionsDAO.createPermission(superAdminPermission);

        List<UserPermission> permissions = permissionsService.getUserPermissions(root);

        assertEquals(1, permissions.size());
        assertEquals(Permission.ADMIN, permissions.get(0).getPermission());
        assertEquals(USER_ID, permissions.get(0).getUser().getId());

        permissionsService.deletePermission(USER_ID, GroupsService.ROOT, Permission.ADMIN);
    }
}
